#!/usr/local/perl/bin/perl

#******************************* COLUMNATOR for EVALITA **********************

#The script take the sentences of the input file annotated in TUT format (see sample a)) and splits the annotated lines 
#of the sentence in a (CoNLL-stile) 10 columns format (see sample b)). The colums are separated by tabs. If several 
#elements compose a single column, they are separated by '|'. Each column can be filled by a token or by '_' 
#(if empty or if the CPOSTAG is NUM (=number)). 
#Observe that, the data included in some column generated by this script may not exactly correspond 
#to those in TUT native. In fact, in order to use the standard characters '|' and '_' as in the CoNLL format, 
#all the characters '|' and '_' in use in the TUT native format have to be converted in other characters 
#(namely '|' in '@' and '_' in '>'). 
#Moreover, the script reduce the native tagset of TUT relations in order to made available an annotation based on 
#the single functional-syntactic component that includes 55 tags (27 primary tags that can be composed with 23 
#secondary tags). 

#Sample:

#a) TUT annotation:
#************** FRASE XXX-333 **************
#1 Ma (MA CONJ COORD ADVERS) [0;TOP-CONJ]
#2 Valona (VALONA NOUN PROPER F £CITY) [3;VERB-SUBJ]
#3 e' (ESSERE VERB MAIN IND PRES INTRANS 3 SING) [1;COORD2ND+ADVERS]
#4 anche (ANCHE ADV CONCESS) [5;ADVB+CONCESS-RMOD]
#5 il (IL ART DEF M SING) [3;VERB-PREDCOMPL+SUBJ]
#6 simbolo (SIMBOLO NOUN COMMON M SING) [5;DET+DEF-ARG]
#7 di (DI PREP MONO) [6;PREP-RMOD]
#8 una (UN ART INDEF F SING) [7;PREP-ARG]
#9 resistenza (RESISTENZA NOUN COMMON F SING) [8;DET+INDEF-ARG]
#10 secolare (SECOLARE ADJ QUALIF ALLVAL SING) [9;ADJC+QUALIF-RMOD]
#11 a (A PREP MONO) [9;PREP-RMOD]
#12 ogni (OGNI ADJ INDEF ALLVAL SING) [11;PREP-ARG]
#13 tentativo (TENTATIVO NOUN COMMON M SING TENTARE INTRANS) [12;DET+QUANTIF-ARG]
#14 di (DI PREP MONO) [13;NOUN-OBJ]
#15 dominazione (DOMINAZIONE NOUN COMMON F SING DOMINARE TRANS) [14;PREP-ARG]
#16 . (#\. PUNCT) [1;END]

#b) 10 colums format:
#1	Ma	MA	CONJ	CONJ	COORD|ADVERS	0	TOP-CONJ	_	_
#2	Valona	VALONA	NOUN	NOUN	PROPER|F|£CITY	3	VERB-SUBJ	_	_
#3	e'	ESSERE	VERB	VERB	MAIN|IND|PRES|INTRANS|3|SING	1	COORD2ND+ADVERS	_	_
#4	anche	ANCHE	ADV	ADV	CONCESS	5	ADVB+CONCESS-RMOD	_	_
#5	il	IL	ART	ART	DEF|M|SING	3	VERB-PREDCOMPL+SUBJ	_	_
#6	simbolo	SIMBOLO	NOUN	NOUN	COMMON|M|SING	5	DET+DEF-ARG	_	_
#7	di	DI	PREP	PREP	MONO	6	PREP-RMOD	_	_
#8	una	UN	ART	ART	INDEF|F|SING	7	PREP-ARG	_	_
#9	resistenza	RESISTENZA	NOUN	NOUN	COMMON|F|SING	8	DET+INDEF-ARG	_	_
#10	secolare	SECOLARE	ADJ	ADJ	QUALIF|ALLVAL|SING	9	ADJC+QUALIF-RMOD	_	_
#11	a	A	PREP	PREP	MONO	9	PREP-RMOD	_	_
#12	ogni	OGNI	ADJ	ADJ	INDEF|ALLVAL|SING	11	PREP-ARG	_	_
#13	tentativo	TENTATIVO	NOUN	NOUN	COMMON|M|SING|TENTARE|INTRANS	12	DET+QUANTIF-ARG	_	_
#14	di	DI	PREP	PREP	MONO	13	NOUN-OBJ	_	_
#15	dominazione	DOMINAZIONE	NOUN	NOUN	COMMON|F|SING|DOMINARE|TRANS	14	PREP-ARG	_	_
#16	.	#\.	PUNCT	PUNCT	_	1	END	_	_

#(The 10 colums have to be read according to the following pattern:
#ID 	FORM 	LEMMA 	CPOSTAG POSTAG 	FEATS 	HEAD 	DEPREL 	PHEAD 	PDEPREL)


@arrayfrase = ();       	
$finefrase = 0;         		
$contit = 0;            		
$filefrasi = '';
while (<>){   
#print "qui\n";
	$test = 0;           	       		
	while ($test == 0){			
		$riga = $_;			
		$stringa = sprintf "%s", $riga;  
		$inizio = substr $stringa, 0,5;      	        
		if ($inizio =~ m/\d/){  
			$stringa =~ s/\_/\@/g; 			
			$stringa =~ s/\|/\>/g;
			push (@arrayfrase,split_in_columns($stringa)); 		        
		}
		else {if ($inizio !~ m/\*/){ 		       
					if ($contit == 0){  	           
						$contit = ++$contit; 	 
						$filefrasi = $filefrasi;#.$titolo;					
					}
	      			$finefrase = ++$finefrase;  
		      	}
				if ($inizio =~ m/\*/){            	
					$titolo = $stringa;     	      	
					if ($finefrase != 0){           	
						$filefrasi = $filefrasi.(join("\n",@arrayfrase))."\n\n";#.$titolo;	
						@arrayfrase = ();			 
						$finefrase = 0;				
					}
	    	  	}
		}	
		$test = ++$test;     			
	}
}
$filefrasi = $filefrasi.(join("\n",@arrayfrase))."\n\n"; 	
print $filefrasi;



######################## SPLIT_IN_COLUMNS ###################################################
#split a single sentence line in 10 columns according to a CoNLL-stile pattern (see sample b) above).

sub split_in_columns{
	my $line = shift(@_);
	my $input_line=$line;
	my $columned_line = '';
	my $id = '';			#first column
	my $form = '';		#second column
	my $lemma = '';	#third column
	my $cpostag = '';	#fourth column
	my $postag = '';		#fifth column
	my $feats = '';		#sixth column	
	my $head = '';		#seventh column
	my $deprel = '';		#eigth column
	my $phead = '';		#nineth column
	my $pdeprel = '';	#tenth column
	$id = substr $line,0,(index $line,' ');
	$line = substr $line,(index $line,' ')+1;
	if ($line !~ m/\(\#\\\( PUNCT/){
			$form = substr $line,0,(index $line,'(')-1;
	}
	else{$form = "(";}
#print $form,"=forma1\n";
	$form =~ s/\s/\|/g;	
#print $form,"=forma2\n";
	if ($line !~ m/\(\#\\\( PUNCT/){
			$line = substr $line,(index $line,'(')+1;
	}
	else{$line = substr $line,(index $line,'(')+3;
	}
#print $line,"=line\n";
	$lemma =  substr $line,0,(index $line,' ');
	$line = substr $line,(index $line,' ')+1;
	$cpostag =  substr $line,0,(index $line,' ');
	$line = substr $line,(index $line,' ')+1;
	$feats = substr $line,0,(index $line,')');
	if ($cpostag =~ m/\)/){	#if there aren't feats associated with cpostag, cpostag includes a ')'
		$cpostag = substr $cpostag,0,(index $cpostag,')');
		$feats = '_';
		$line = substr $line,(index $line,'[')+1;
	}
	else{$line = substr $line,(index $line,')')+3;
	}
	$feats =~ s/\s/\|/g;	#since feats can include spaces, they have to be substituted with a character
	$postag = $cpostag;
	if ($line !~ m/\;/){	#if line is unannotated (ex. marker) begins with ']'
		$head = '_';
		$deprel = '_';
	}
	else{$head = substr $line,0,(index $line,';');
			$line = substr $line,(index $line,';')+1;
			$deprel = substr $line,0,(index $line,']');
			if ($deprel !~ m/\//){
					$deprel = isola_syntcompo($deprel);
			}
			else{$posslash = index $deprel, '/';
					$deprel1 = substr $deprel, 0, $posslash;
					$deprel2 = substr $deprel, ($posslash + 1);
					$deprel = (isola_syntcompo($deprel1)).'/'.(isola_syntcompo($deprel2));
			}
	}
	if ($cpostag eq 'NUM'){	#if CPOSTAG is NUM(ber) the feaures are _ (instead of the same as form as in native TUT)
		$feats = '_';
	}
	$phead = '_';
	$pdeprel = '_';
	$columned_line = $id.'&'.$form.'&'.$lemma.'&'.$cpostag.'&'.$postag.'&'.$feats.'&'.$head.'&'.$deprel.'&'.$phead.'&'.$pdeprel;
	$columned_line =~ s/&/\t/g; 
return $columned_line;
}






sub isola_syntcompo{
	my $syntcompo = shift(@_);
	my $first = '';
	my $second = '';
	my $third = '';
	my $postrat1 = '';
	my $posslash = '';
	if ($syntcompo =~ m/\-/){	#se deprel contiene piu' componenti, isola quella sintattica solamente da mettere in colonne
			$postrat1 = index $syntcompo, '-';	       #isola la prima componente
#print $postrat1,"=postrat1\n";
			$first = substr $syntcompo, 0, $postrat1;      #che precede il primo trattino
#print $first,"=first\n";
			$second = substr $syntcompo, ($postrat1 + 1);  #e la parte che segue il primo trattino
#print $second,"=second\n";
			if ($second =~ m/-/){                          #se $second contiene trattino allora la relazione ha 3 componenti (morfo, sint e sem)
	 				$third = substr $second, ((index $second,'-') + 1); #la terza e' la semantica, la seconda la sintattica e la prima morfologica
	 				$second = substr $second, 0, (index $second, '-');         		
					$syntcompo = $second;
#print $third,"=third\n";
			}
			else{       #altrimenti ($second non contiene trattini e non va scomposto)
					if (($first =~ m/ADJ/) or ($first =~ m/ADV/) or ($first =~ m/ART/) or ($first =~ m/CONJ/) or ($first =~ m/DATE/) or
 						($first =~ m/DET/) or ($first =~ m/NUM/) or ($first =~ m/PRON/) or ($first =~ m/VERB/) or
		 				($first =~ m/NOUN/) or ($first =~ m/PREP/) or ($first =~ m/PDET/)){    #se $first `e la componente morfologica
								$syntcompo = $second;
     				}	
     				else{$syntcompo = $first; #altrimenti la prima e' la sintattica e la seconda semantica
     				}
   			}
     }
return $syntcompo;
}
